use Test::More tests => 3;
use strict;
use warnings;

use FindBin qw/$Bin/;
use lib "$Bin/lib";
use NAR::Helper;

my $helper = NAR::Helper->new;

ok $helper->connect, "connected";
ok $helper->channel_open, "channel_open";

ok $helper->basic_qos( { prefetch_count => 5 } ), "qos";

END {
    $helper->channel_close;
    $helper->disconnect;
}
